@@ -92,7 +92,7 @@ def option_cc(args):
 
     if '-h' in args:
         help = f"""
-        {utils.CYELLOW}-cc{utils.CEND} 
+        {utils.CYELLOW}-cc{utils.CEND}       {utils.CGREEN}InCli -u "Org alias" -cc {utils.CEND}
             Checks the catalogs. Gets all catalogs, does a getOffers, getOfferDetails. Performs basketwithoutconfig and basket with config, if -basket is set. 
             {utils.CYELLOW}-code{utils.CEND} - Catalogue Code. does it for a single catalog. 
             {utils.CYELLOW}-list{utils.CEND} - List all the catalogues. 
@@ -291,7 +291,7 @@ def option_logs(args):
             Modifiers:
             {utils.CYELLOW}-loguser field:value{utils.CEND}, filter the logs for the specified user. The user can be specified by any field in the User Object. 
                 {utils.CGREEN}-loguser Id:0053O000000IHneQAG, -loguser "name:Onboarding Site Guest User", -loguser Alias:John.Doe, -loguser FirstName:Onboarding, -loguser ProfileId:00e3O000000IHneQAG{utils.CEND}
-                {utils.CGREEN}Defaultable loguser{utils.CEND}: Can be set to default. {utils.CGREEN}-default:set loguser "Alias:John.Doe{utils.CEND}"
+                {utils.CLIGHT_PURPLE}Defaultable loguser{utils.CEND}: Can be set to default. {utils.CGREEN}-default:set loguser "Alias:John.Doe{utils.CEND}"
             {utils.CYELLOW}-where X {utils.CEND}--> the where clause for a query.  
                 {utils.CGREEN}InCli -logs -loguser Alias:John.Doe -where="Status<>'Success" -last 10" {utils.CEND} --> parses the last 10 logs for user John.Doe where the status is not Success
             {utils.CYELLOW}-file {utils.CEND}--> creates 2 files for the parsed file, txt and html format. 
@@ -316,7 +316,6 @@ def option_logs(args):
     store = True if '-store' in args else False
     error = True if '-error' in args else False
 
-
     if loguser == None:
         loguser = restClient.getConfigVar('loguser')
 
@@ -324,15 +323,41 @@ def option_logs(args):
         connectionInit(args)
         lim = 50 if limit == None else limit
 
-        debugLogs.printLogRecords(loguser=loguser,limit=lim,whereClause=whereClause)
-        return
+        return debugLogs.printLogRecords(loguser=loguser,limit=lim,whereClause=whereClause)
 
-    logUserId = None
-    if inputfile == None:
+    parseContext = {
+        'logId' :logId,
+        'filepath':inputfile,
+        'printLimits':limitInfo,
+        'lastN':last,
+        'loguser':loguser,
+        'level':level,
+        'whereClause':whereClause,
+        'writeToFile':toFile,
+        'tail':tail,
+        'deleteLogs':deletelogs,
+        'parseStore':store,
+        'logToStore':True,
+        'onlyErrors':error,
+        'operation':None
+    }
+
+    if tail:
         connectionInit(args)
+        return debugLogs.parseTail(parseContext)
 
-   # debugLogs.parseLog(logId=logId,lastN=last,level=level,whereClause=whereClause,writeToFile=toFile,filepath=inputfile,tail=tail,deleteLogs=deletelogs)
-    return debugLogs.parseLog(logId=logId,filepath=inputfile,loguser=loguser,lastN=last,level=level,whereClause=whereClause,writeToFile=toFile,tail=tail,deleteLogs=deletelogs,store=store,error=error,printLimits=limitInfo)
+    elif store:
+        return debugLogs.parseStorage(parseContext)
+
+    elif inputfile != None:
+        return debugLogs.parseFile(parseContext)
+
+    elif logId != None:  #check if already downloaded        
+        connectionInit(args)
+        return debugLogs.parseLogId(logId,parseContext)
+    else:
+        connectionInit(args)
+        return debugLogs.parseLogs_LastN(parseContext)
 
 def option_h(args):
 
@@ -344,7 +369,7 @@ def option_h(args):
     for f in functions:
         if f == 'option_h':
             continue
-
+        args = '-h'
         print(eval(f'{f}(args)'))      
     print()
 
@@ -391,7 +416,7 @@ def _main(argsIn):
 
         help = f"""
         {utils.CYELLOW}-u{utils.CEND} --> username or org alias to be used to log into the org. 
-            {utils.CGREEN}Defaultable u{utils.CEND}: Can be set to a default value --> {utils.CGREEN}InCli -default:set u "xx@adas.com {utils.CEND}"""
+            {utils.CLIGHT_PURPLE}Defaultable u{utils.CEND}: Can be set to a default value --> {utils.CGREEN}InCli -default:set u "xx@adas.com {utils.CEND}"""
         print(help)
 
     funcs = getmembers(sys.modules[__name__], isfunction)
@@ -410,7 +435,8 @@ def _main(argsIn):
         #       module = __import__('InCli')
             res = None
             res = eval(f'{ar}(args)')
-            return res
+            if '-h' not in argsIn:
+                return res
         #      func = getattr(module, ar)
         #      func(args)
 
