@@ -4,7 +4,7 @@ import simplejson,logging
 import colorama
 import sys,time,os
 import ansi2html,re
-import threading
+import threading,traceback
 from queue import Queue
 
 def _queryLogRecords(logUserId=None,limit=50,whereClause=None):
@@ -45,7 +45,8 @@ def _queryLogData(logId):
     logbody = restClient.callAPI(action)
     return logRecord,logbody
 
-def _parseStorage(error=False,printLimits=False):
+def parseStorage(pc):
+    
     search_dir = restClient.logFolder()
 
     os.chdir(search_dir)
@@ -53,14 +54,20 @@ def _parseStorage(error=False,printLimits=False):
     files = [os.path.join(search_dir, f) for f in files] # add path to each file
     files.sort(key=lambda x: os.path.getmtime(x))
     fileNames = [os.path.basename(f) for f in files]
+    fileNames = [f for f in fileNames if '.log' in f]
+
     ids = [f.split('.')[0] for f in fileNames]
     print(f"Parsing {len(ids)} files in {search_dir}")
-    for num,id in enumerate(ids):
-        sys.stdout.write("\r%d%%" % int(100*num/len(ids)))
-        res = parseLog(logId=id,error=error,printLimits=printLimits)
-        
 
-def _parseTail(loguser=None,level=None,whereClause=None,deleteLogs=True,printLimits=False):
+    try:
+        _parseIds(ids,pc,printProgress=True,printNum=False)
+
+    except KeyboardInterrupt:
+        print('Interrupted')
+    
+    _printParsingResults(pc)
+
+def parseTail(pc):
     deleteList = []
     def deleteRecords(q):
       while True:
@@ -81,8 +88,8 @@ def _parseTail(loguser=None,level=None,whereClause=None,deleteLogs=True,printLim
 
     waitingPrinted = False
     q= None
-    restClient.glog().debug(f"deleteLogs-->{deleteLogs}")
-    if deleteLogs==True:
+    restClient.glog().debug(f"deleteLogs-->{pc['deleteLogs']}")
+    if pc['deleteLogs']==True:
         if loguser==None:
             username = restClient.getCurrentThreadConnection()['username']
             loguser = f"Username:{username}"
@@ -92,33 +99,35 @@ def _parseTail(loguser=None,level=None,whereClause=None,deleteLogs=True,printLim
         threading.Thread(target=deleteRecords,args=(q,), daemon=True).start()
         threading.Thread(target=deleteRecords,args=(q,), daemon=True).start()
     
-    logUserId = _getLogUserId(loguser=loguser) if loguser != None else None
+    logUserId = _getLogUserId(loguser=pc['loguser']) if pc['loguser'] != None else None
 
     try:
         while (True):
             where = f" {timefield} > {_timez} "
-            where = f" {whereClause} and {where}" if whereClause is not None else where
+            where = f" {pc['whereClause']} and {where}" if pc['whereClause'] is not None else where
             where = f" logUserId='{logUserId}'and {where} " if logUserId is not None else where
 
             fields = "Id,LogUserId,LogLength,LastModifiedDate,Request,Operation,Application,Status,DurationMilliseconds,StartTime,Location,RequestIdentifier"
-            logRecords = query.queryRecords(f"Select {fields} FROM ApexLog where {where} ")
+            logRecords = query.queryRecords(f"Select {fields} FROM ApexLog where {where} order by StartTime desc")
             if len(logRecords) > 0:
                 waitingPrinted = False
 
-                for record in logRecords:
+                logIds = [record['Id'] for record in logRecords]
+                _parseIds(logIds=logIds,pc=pc,raiseKeyBoardInterrupt=True)
+
+                _time = logRecords[-1][timefield]
+                _timez = _time.split('.')[0] + "Z"
+
+                if q!=None:
+                    for logId in logIds:
+                        q.put(logId)
+                        restClient.glog().debug(f"{logId} into queue...")
 
-                    parseLog(logId=record['Id'], loguser=loguser,level=level,printLimits=printLimits)                
-                    _time = record[timefield]
-                    _timez = _time.split('.')[0] + "Z"
-                    if q!=None:
-                        q.put(record['Id'])
-                        restClient.glog().debug(f"{record['Id']} into queue...")
-                    
             else:
                 if waitingPrinted == False:
                     print()
-                    if loguser != None:
-                        print(f"waiting for debug logs for user {loguser}")
+                    if pc['loguser'] != None:
+                        print(f"waiting for debug logs for user {pc['loguser']}")
                     else:
                         print(f"waiting for debug logs ")
 
@@ -127,6 +136,7 @@ def _parseTail(loguser=None,level=None,whereClause=None,deleteLogs=True,printLim
             time.sleep(2)
     except KeyboardInterrupt as e:
         print()
+        _printParsingResults(pc)
         print("Terminating -tail..., cleaning up")
         if q != None:
             while q.empty()==False:
@@ -134,7 +144,91 @@ def _parseTail(loguser=None,level=None,whereClause=None,deleteLogs=True,printLim
         print('Terminated')
         return
 
-def _parseLogs_LastN(lastN,loguser=None,whereClause=None,printLimits=True,writeToFile=False):
+def _parseIds(logIds,pc,raiseKeyBoardInterrupt=False,printProgress=False,threads=10,printNum=True):
+    def readBody(q):
+      while True:
+        Id = q.get()
+        _readBodyId(Id)
+        restClient.glog().debug(f"Read body for Id {Id}")
+        q.task_done()
+
+    if 'total' not in pc:
+        pc['total'] = 0
+    if 'parsed' not in pc:
+        pc['parsed'] = []
+    if 'errors' not in pc:
+        pc['errors'] = []
+    parsed = pc['parsed']
+    errors = pc['errors']
+
+    if threads >0 and ('queue' not in pc or pc['queue'] == False):
+        q = Queue(maxsize=0)
+        for x in range(0,threads):
+            threading.Thread(target=readBody,args=(q,), daemon=True).start()
+    if threads>0:
+        for logid in logIds:
+            q.put(logid)
+            
+    for num,logId in enumerate(logIds):
+        if printProgress:
+            sys.stdout.write("\r%d%%" % int(100*num/len(logIds)))
+        try:
+            _parsed={
+                'logId':logId,
+                'status':'ok'
+            }
+            parsed.append(_parsed)
+            pc['logId'] = logId
+            _readBody(pc)
+            _parseLog_print(pc)
+            _printParsedLog(pc)
+            if printNum:
+                print( pc['total']+num+1)
+            if pc['context']['exception'] == True:
+                _parsed['status'] = pc['context']['exception_msg'][0:200]
+
+        except KeyboardInterrupt:
+            if raiseKeyBoardInterrupt:
+                raise
+            break
+        except utils.InCliError as e:
+             _parsed['status'] = f"Parse error: {e.args[0]['errorCode']}  "
+             utils.printException(e)
+             errors.append(e)
+        except Exception as e:
+            _parsed['status'] = f"Unknown: {e}"
+            errors.append(e)
+            print(traceback.format_exc())
+
+
+    pc['total'] = pc['total'] + num + 1
+    
+def _printParsingResults(pc):
+    print()
+
+    if 'parsed' not in pc:
+        print("No files parsed.")
+        return 
+    parsed = pc['parsed']
+    errors = pc['errors']
+
+    print(f"{pc['total']} logs parsed")
+    parsed = [par for par in parsed if par['status']!='ok']
+
+    if len(parsed) == 0:
+        print("No errors.")
+    if len(parsed)>0:
+        utils.printFormated(parsed)
+
+        errors = list({error.args[0]['errorCode']:error for error in errors}.values())
+        for error in errors:
+            utils.printException(error)  
+
+def parseLogs_LastN(pc):
+    whereClause = pc['whereClause']
+    loguser = pc['loguser']
+    lastN = pc['lastN']
+
     where = f" where {whereClause} " if whereClause is not None else ''
     where = f" where logUserId='{_getLogUserId(loguser)}' " if loguser is not None else where
 
@@ -144,8 +238,9 @@ def _parseLogs_LastN(lastN,loguser=None,whereClause=None,printLimits=True,writeT
     logIds = query.queryFieldList(f"Select Id FROM ApexLog {where} order by LastModifiedDate desc limit {lastN}")
     if logIds == None or len(logIds)==0:
         utils.raiseException(errorCode='NO_LOG',error=f'No logs can be found. ',other=q)
-    for logId in logIds:
-        parseLog(logId,printLimits=printLimits,writeToFile=writeToFile)
+
+    _parseIds(logIds,pc)
+    _printParsingResults(pc)
 
 def _getLogUserId(loguser):
     chunks = loguser.split(":")
@@ -158,29 +253,88 @@ def _getLogUserId(loguser):
         utils.raiseException('QUERY',f"User with field {key} = {value} does not exist in the User Object.")    
     return id
 
+def _readBodyId(logId):
+    filename = f"{restClient.logFolder()}{logId}.log"
+
+    if file.exists(filename) == True:
+        body = file.read(filename)
+
+        if len(body)==0:
+            print("The file seems corrupted. Getting log from server.")
+            file.delete(filename)
+            return _readBodyId(logId)
+        return None,body,filename
+    else:
+        logRecord,body = _queryLogData(logId) 
+        body = _logRecord_toString(logRecord=logRecord) + body  
+        _saveToStore(logId,body)
+        return logRecord,body,filename
+
+def _readBody(pc):
+    logId = pc['logId']
+    pc['logRecord'],pc['body'],pc['filepath'] = _readBodyId(logId)
+
+def parseFile(parseContext):
+    parseContext['body'] = file.read(parseContext['filepath'])
+    parseContext['operation'] = 'parsefile'
+    name = os.path.basename(parseContext['filepath']).split('.')[0]
+    parseContext['logId']=name
+    context =  _parseLog_print(parseContext)
+    _printParsedLog(parseContext)
+    return context
+
+def parseLogId(logId,parseContext):
+    _readBody(parseContext)
+    if parseContext['body'] == None :
+        utils.raiseException(errorCode='NO_LOG',error=f'The requested log <{logId}> cannot be found. ')
+
+    if len(parseContext['body'])==0:
+        utils.raiseException(errorCode='NO_LOG',error=f'The body for the requested log <{logId}> is empty. ')
+    context =  _parseLog_print(parseContext)
+    _printParsedLog(parseContext)
+
+    return context
+
 def parseLog(logId=None,filepath=None,printLimits=False,lastN=1,loguser=None,level=None,whereClause=None,writeToFile=False,tail=False,deleteLogs=False,store=False,error=False):
-    body = None
+    parseContext = {
+        'logId' :logId,
+        'filepath':filepath,
+        'printLimits':printLimits,
+        'lastN':lastN,
+        'loguser':loguser,
+        'level':level,
+        'whereClause':whereClause,
+        'writeToFile':writeToFile,
+        'tail':tail,
+        'deleteLogs':deleteLogs,
+        'parseStore':store,
+        'logToStore':True,
+        'onlyErrors':error,
+        'operation':None
+    }
 
     if tail == True:
-        return _parseTail(loguser=loguser,level=level,whereClause=whereClause,deleteLogs=deleteLogs)
+        return parseTail(parseContext)
 
     elif store == True:
-        return _parseStorage(error=error,printLimits=printLimits)
+        return parseStorage(parseContext)
 
     elif filepath != None:
-        body = file.read(filepath)
+        return parseFile(parseContext)
 
     elif logId != None:  #check if already downloaded
-        filename = f"{restClient.logFolder()}{logId}.log"
-        if file.exists(filename) == True:
-            body = file.read(filename)
-            filepath = filename
-        else:
-           logRecord,body = _queryLogData(logId) 
-           body = _logRecord_toString(logRecord=logRecord) + body
-
+        return parseLogId(logId,parseContext)
     else:
-        return _parseLogs_LastN(lastN,loguser,whereClause=whereClause,printLimits=printLimits,writeToFile=writeToFile)
+        return parseLogs_LastN(parseContext)
+
+
+def _saveToStore(logId,body):
+    filename = f"{restClient.logFolder()}{logId}.log"
+    file.write(filename,body) 
+
+def _parseLog_print(pc):
+    body = pc['body']
+    logId = pc['logId']
 
     if body == None :
         utils.raiseException(errorCode='NO_LOG',error=f'The requested log <{logId}> cannot be found. ')
@@ -188,11 +342,7 @@ def parseLog(logId=None,filepath=None,printLimits=False,lastN=1,loguser=None,lev
     if len(body)==0:
         utils.raiseException(errorCode='NO_LOG',error=f'The body for the requested log <{logId}> is empty. ')
 
-    if filepath == None:
-        filename = f"{restClient.logFolder()}{logId}.log"
-        file.write(filename,body)
-
-    return parse(logId,body,printLimits=printLimits,level=level,writeTofile=writeToFile,error=error,limits=printLimits)
+    return parse(pc)    
 
 def printLogRecords(loguser=None,limit=50,whereClause=None):
     logUserId = _getLogUserId(loguser) if loguser != None else None
@@ -209,6 +359,7 @@ def printLogRecords(loguser=None,limit=50,whereClause=None):
         logs1.append(log)
 
     utils.printFormated(logs1,rename="LogLength%Len:DurationMilliseconds%ms:Application%App")
+    return logs
 
 def delta(obj,field):
     return obj[field][1] - obj[field][0] if len(obj[field]) > 1 else 0
@@ -245,10 +396,6 @@ def setTimes(context,line,obj=None,field=None,value=None,chunkNum=None,type=None
         addList(obj,'timeStamp',0)
 
 
-  #  if _context['exception'] == True and obj['exception'] == False:
-  #      obj['exception'] = True
-  #      _context['exception'] = False
-
     if obj['type'] is None:
         print()
 
@@ -293,22 +440,29 @@ def createContext():
     context['firstLineOut'] = True
     return context
 
-def parse(logId,logData,printLimits=True,userDebug=True,level=None,writeTofile=False,error=False,limits=False):
+def parse(pc):
+   # logId,logData,printLimits=True,userDebug=True,level=None,writeTofile=False,error=False,limits=False
+    logId = pc['logId']
+
     try:
-       return  _parse(logId,logData,printLimits,userDebug,level,writeTofile,error,limits)
+       return  _parse(pc)
     except KeyboardInterrupt as e:
         print(f"Parsing for logI {logId} interrupted.")
+        raise
     except Exception as e:
+        print(f"Exception while parsing for logI {logId} ")
         raise 
 
-def _parse(logId,logData,printLimits=True,userDebug=True,level=None,writeTofile=False,error=False,limits=False):
-    context = createContext()
-    context['printLimits'] = printLimits
-    context['userDebug'] = userDebug
-    context['logLevel'] = level
-    context['logId'] = logId
-    context['limits'] = limits
+def _printParsedLog(pc):
+    context = pc['context']
+    context['debugList'] = processRepetition(context['debugList'])
+    printDebugList(pc)
 
+def _parse(pc):
+    context = createContext()
+    pc['context'] = context
+
+    logData = pc['body']
     context['lines'] = logData.splitlines()
     debugList = []
 
@@ -316,8 +470,6 @@ def _parse(logId,logData,printLimits=True,userDebug=True,level=None,writeTofile=
     context['openItemsList'] = []
 
     for num,line in enumerate(context['lines']):
-   #     print(num)
-
         chunks = line.split('|')
         context['chunks'] = chunks
         context['lenChunks'] = len(chunks)
@@ -325,6 +477,9 @@ def _parse(logId,logData,printLimits=True,userDebug=True,level=None,writeTofile=
         context['line'] = line
         context['num'] = num
 
+        if '16994953586' in chunks[0]:
+            a=1
+
         if context['firstLineIn'] == True:
             if 'APEX_CODE' in line:
                 context['firstLineIn'] = False
@@ -333,6 +488,8 @@ def _parse(logId,logData,printLimits=True,userDebug=True,level=None,writeTofile=
                     'type':'LOGDATA',
                     'output':levels
                 }
+                debugList.append(obj)
+
                 continue      
             else:
                 obj = {
@@ -342,12 +499,10 @@ def _parse(logId,logData,printLimits=True,userDebug=True,level=None,writeTofile=
                 debugList.append(obj)
                 continue
 
-
-
         if context['lenChunks'] == 1:
             continue
 
-        if len(chunks)>1 and chunks[1] in ['HEAP_ALLOCATE','STATEMENT_EXECUTE','VARIABLE_SCOPE_BEGIN','HEAP_ALLOCATE','SYSTEM_METHOD_ENTRY','SYSTEM_METHOD_EXIT','SOQL_EXECUTE_EXPLAIN']:
+        if len(chunks)>1 and chunks[1] in ['HEAP_ALLOCATE','STATEMENT_EXECUTE','VARIABLE_SCOPE_BEGIN','HEAP_ALLOCATE','SYSTEM_METHOD_ENTRY','SYSTEM_METHOD_EXIT','SOQL_EXECUTE_EXPLAIN','ENTERING_MANAGED_PKG','SYSTEM_CONSTRUCTOR_ENTRY','SYSTEM_CONSTRUCTOR_EXIT']:
             continue
 
         if '|SYSTEM_MODE_EXIT|' in line:
@@ -391,8 +546,8 @@ def _parse(logId,logData,printLimits=True,userDebug=True,level=None,writeTofile=
     if len(context['openItemsList']) > 0:
         a=1
     appendEnd(context)
-    context['debugList'] = processRepetition(debugList)
-    printDebugList(context,logId,writeTofile,error=error)
+ #   context['debugList'] = processRepetition(debugList)
+ #   printDebugList(context)
 
     return context
 
@@ -403,10 +558,13 @@ def append_and_increaseIdent(context,obj,increase=True):
     context['debugList'].append(obj)
 
 def decreaseIdent_pop_setFields(context,type,value,key='key',endsWith=None,decrease=True):
-    if decrease == True:
-        decreaseIdent(context)
     obj = popFromList(context,type=type,key=key,value=value,endsWith=endsWith)
-    setTimes(context,context['line'],obj)
+    if obj == None:
+        a=1
+    else:
+        if decrease == True:
+            decreaseIdent(context)
+        setTimes(context,context['line'],obj)
     return obj
 
 def parseUserInfo(context):
@@ -436,6 +594,14 @@ def parseSOQL(context):
         obj['query'] = chunks[4]
         obj['object'] = chunks[4].lower().split(' from ')[1].strip().split(' ')[0]
         obj['apexline'] = chunks[2][1:-1]
+
+        soql = obj['query'].lower()
+        _from = soql.split(' from ')[-1].strip()
+        _from = _from.split(' ')[0]
+
+        obj['from'] = _from
+        obj['output'] = f"Select: {obj['from']} --> No SOQL_EXECUTE_END found"
+
         append_and_increaseIdent(context,obj,increase=False)
         return True
 
@@ -443,13 +609,13 @@ def parseSOQL(context):
  #   if '|SOQL_EXECUTE_END|' in line:
         context['totalQueries'] = context['totalQueries'] + 1
         obj = decreaseIdent_pop_setFields(context,type="SOQL",key='type',value='SOQL',decrease=False)
+   #     if obj == None:
+   #         return True
+   #     print(chunks[0])
+    #    if '30536880065' in chunks[0]:
+    #        a=1
         obj['rows'] = chunks[3].split(':')[1]
-
-        soql = obj['query'].lower()
-        _from = soql.split(' from ')[-1].strip()
-        _from = _from.split(' ')[0]
-
-        obj['output'] = f"Select: {_from} --> {obj['rows']} rows"
+        obj['output'] = f"Select: {obj['from']} --> {obj['rows']} rows"
         return True
 
     return False
@@ -463,7 +629,7 @@ def parseLimits(context):
 
     if '|LIMIT_USAGE_FOR_NS|' in line:
         obj = setTimes(context,line,type='LIMIT')
-        obj['output'] = f"{chunks[1].lower()}  {chunks[1]}"
+        obj['output'] = f"{chunks[1].lower()}  {chunks[2]}"
         context['debugList'].append(obj)
 
         limits = chunks[2]
@@ -551,18 +717,22 @@ def parseUserDebug(context):
         obj['type'] = 'DEBUG'
         obj['subType'] = chunks[3]
         obj['string'] = chunks[4]
-        obj['apexline'] = chunks[2][1:-1]
         obj['output'] = obj['string'] 
+        if obj['subType'] == 'ERROR':
+            context['exception'] = True
+            context['exception_msg'] = obj['output']
+        obj['apexline'] = chunks[2][1:-1]
         context['debugList'].append(obj)
-        next = 1
-        nextline = context['lines'][context['num']+next]
-        while '|' not in nextline:
-            obj = context['debugList'][-1].copy()
-            context['debugList'].append(obj)
-            obj['string'] = nextline
-            obj['output'] = nextline
-            next = next + 1
+        if context['num']<(len(context['lines'])-1):
+            next = 1
             nextline = context['lines'][context['num']+next]
+            while '|' not in nextline:
+                obj = context['debugList'][-1].copy()
+                context['debugList'].append(obj)
+                obj['string'] = nextline
+                obj['output'] = nextline
+                next = next + 1
+                nextline = context['lines'][context['num']+next]
 
         if '*** getCpuTime() ***' in line:
             chs = chunks[4].split(' ')
@@ -613,14 +783,17 @@ def parseExceptionThrown(context):
     if context['lenChunks']>1 and chunks[1] == 'EXCEPTION_THROWN':
         obj = setTimes(context,line,type='EXCEPTION',field='output',value=chunks[3])
         context['exception'] = True
+        context['exception_msg'] = obj['output']
+
         context['debugList'].append(obj)
         context['file_exception'] = True
         next = 1
         nextline = context['lines'][context['num']+next]
         while '|' not in nextline:
-            obj = context['debugList'][-1].copy()
-            context['debugList'].append(obj)
-            obj['output'] = nextline
+            if nextline != '':
+                obj = context['debugList'][-1].copy()
+                context['debugList'].append(obj)
+                obj['output'] = nextline
             next = next + 1
             nextline = context['lines'][context['num']+next]
         return True
@@ -628,14 +801,17 @@ def parseExceptionThrown(context):
     if context['lenChunks']>1 and chunks[1] == 'FATAL_ERROR':
         obj = setTimes(context,line,type='EXCEPTION',field='output',value=chunks[2])
         context['exception'] = True
+        context['exception_msg'] = obj['output']
+
         context['debugList'].append(obj)
         context['file_exception'] = True
         next = 1
         nextline = context['lines'][context['num']+next]
         while '|' not in nextline:
-            obj = context['debugList'][-1].copy()
-            context['debugList'].append(obj)
-            obj['output'] = nextline
+            if nextline != '':
+                obj = context['debugList'][-1].copy()
+                context['debugList'].append(obj)
+                obj['output'] = nextline
             next = next + 1
             nextline = context['lines'][context['num']+next]
         return True
@@ -693,8 +869,7 @@ def parseWfRule(context):
 def parseMethod(context):
     line = context['line']
     chunks = context['chunks'] 
-    if context['lenChunks']>1 and '|METHOD_' in  chunks[1]:
-  #  if '|METHOD_' in line:
+    if context['lenChunks']>1 and 'METHOD_' in  chunks[1]:
         if len(chunks)<4:
             print(line)
             return
@@ -753,11 +928,10 @@ def parseVariableAssigment(context):
 
         else:
             context['EXP_VAR'] = False
-        return True
+        return False
 
     if _isInOp(context,'VARIABLE_ASSIGNMENT'):
         if len(chunks) >= 5:
-
             if 'ExecutionException' in chunks[4] or 'ExecutionException' in chunks[4]:
                 obj = setTimes(context,line,type='VAR_ASSIGN')
                 obj['type'] = 'VAR_ASSIGN'
@@ -813,6 +987,8 @@ def parseConstructor(context):
     if _isInOp(context,'CONSTRUCTOR_ENTRY'):
   #  if '|CONSTRUCTOR_ENTRY|' in line:
         obj = setTimes(context,line,field='output',value=chunks[5],type='CONSTRUCTOR')
+        obj['apexline'] = chunks[2][1:-1] if chunks[2]!='[EXTERNAL]' else 'EX'
+
         append_and_increaseIdent(context,obj)
         return True
 
@@ -868,15 +1044,18 @@ def parseFlow(context):
     if _isInOp(context,'FLOW_START_INTERVIEW_BEGIN'):
   #  if '|FLOW_START_INTERVIEW_BEGIN|' in line:
         obj = setTimes(context,line,type='FLOW_START_INTERVIEW')
-        obj['Id'] = chunks[2]
+        obj['interviewId'] = chunks[2]
         obj['Name'] = chunks[3]
         obj['output'] = obj['Name']
-        obj['key'] = obj['Id']
+      #  obj['key'] = obj['Id']
         append_and_increaseIdent(context,obj)
+        return True
 
     if _isInOp(context,'FLOW_START_INTERVIEW_END'):
  #   if '|FLOW_START_INTERVIEW_END|' in line:
-        decreaseIdent_pop_setFields(context,'FLOW_START_INTERVIEW',value=chunks[2])
+        interviewId = chunks[2]
+        decreaseIdent_pop_setFields(context,'FLOW_START_INTERVIEW',key='interviewId',value=interviewId)
+        return True
 
     if _isInOp(context,'FLOW_ELEMENT_ERROR'):
   #  if '|FLOW_ELEMENT_ERROR|' in line:
@@ -886,20 +1065,56 @@ def parseFlow(context):
         obj['elementName'] = chunks[4]
         obj['output'] = utils.CRED+ f"{obj['message']} in {obj['elementType']}:{obj['elementName']}" + utils.CEND
         debugList.append(obj)
-        context['file_exception'] = True
+        context['exception'] = True
+        context['exception_msg'] = obj['output']
+        return True
+    
+    if _isInOp(context,'FLOW_ELEMENT_BEGIN'):
+        obj = setTimes(context,line,type='FLOW_ELEMENT')
+        obj['interviewId'] = chunks[2]
+        obj['elementType'] = chunks[3]
+        obj['elementName'] = chunks[4]
+        obj['output'] = f"{obj['elementType']}-{obj['elementName']}"
+        append_and_increaseIdent(context,obj)
+        return True
+
+    if _isInOp(context,'FLOW_ELEMENT_END'):
+        interviewId = chunks[2]
+        decreaseIdent_pop_setFields(context,'FLOW_ELEMENT',key='interviewId',value=interviewId)
+
+    if _isInOp(context,'FLOW_RULE_DETAIL'):
+        values = {
+            'type':'FLOW_ELEMENT',
+            'elementType':'FlowDecision',
+            'interviewId':chunks[2],
+            'elementName':chunks[3]
+        }
+        obj = getFromDebugList(context,values)
+      #  obj = setTimes(context,line,type='FLOW_RULE_DETAIL')
+      #  obj['interviewId'] = chunks[2]
+        obj['ruleName'] = chunks[3]
+        obj['result'] = chunks[4]
+        obj['output'] = f"{obj['elementType']}-{obj['elementName']} -- {obj['ruleName']}->{obj['result']}"
+     #   append_and_increaseIdent(context,obj,increase=False)
+        return True
+
+    return False
 
 def escape_ansi(line):
     ansi_escape =re.compile(r'(\x9B|\x1B\[)[0-?]*[ -\/]*[@-~]')
     return ansi_escape.sub('', line)
 
-def printDebugList(context,logId,toFile=False,error=False):
+def printDebugList(pc):
+    logId = pc['logId']
+    toFile= pc['writeToFile'] if 'writeToFile' in pc else False
+
     if toFile == True:
         filename = f"{restClient.logFolder()}{logId}_ansi.txt"
 
         original_stdout = sys.stdout
         with open(filename, 'w') as f:
             sys.stdout = f 
-            _printDebugList(context,logId)
+            _printDebugList(pc)
             sys.stdout = original_stdout 
         data = file.read(filename)
         html = ansi2html.Ansi2HTMLConverter().convert(data)
@@ -913,10 +1128,13 @@ def printDebugList(context,logId,toFile=False,error=False):
  
     else:
         colorama.just_fix_windows_console()
-        _printDebugList(context,logId,error)
+        _printDebugList(pc)
 
-def _printDebugList(context,logId,error=False):
-    if context['file_exception'] == False and error == True:
+def _printDebugList(pc):
+    logId = pc['logId'] if 'logId' in pc else None
+    context = pc['context']
+    onlyErrors = pc['onlyErrors'] if 'onlyErrors' in pc else False
+    if context['exception'] == False and onlyErrors == True:
         return 
     print('___________________________________________________________________________________________________________________________________________________________________')
     if logId != None:
@@ -935,14 +1153,15 @@ def _printDebugList(context,logId,error=False):
                 firstLines = False
                 print()
                 print()
-        if context['limits'] == False:
+        printLimits = pc['printLimits'] if 'printLimits' in pc else False
+        if printLimits == False:
             if '*** getCpuTime() ***' in d['output']:
                 continue
             if '*** getQueries() ***' in d['output']:
                 continue
             if d['type'] == 'LIMIT':
                 continue
-        printParsedLog(context,d)    
+        printParsedLog(pc,d)    
     print()
 def processRepetition(debugList):
     def isRep(repss,x,parsedLine):
@@ -1028,8 +1247,8 @@ def rootString(context):
 
     return str     
 
-def printParsedLog(context,d):
-
+def printParsedLog(pc,d):
+    context = pc['context']
     Cinit = utils.CEND
 
     _plimit=' '
@@ -1046,8 +1265,9 @@ def printParsedLog(context,d):
     if 'ident' not in d:
         print()
     level = d['ident']
-    if context['logLevel'] != None:
-        if level > int(_context['logLevel']):
+    pcLevel = pc['level'] if 'level' in pc else None
+    if pcLevel != None:
+        if level > int(context['parseContext']['level']):
             return
 
     #colors
@@ -1067,7 +1287,7 @@ def printParsedLog(context,d):
     elif d['type'] == 'DML':
         Cinit =  utils.CYELLOW
     elif d['type'] == 'CODE_UNIT':
-        Cinit =  utils.CPURPLE
+        Cinit =  utils.CYELLOW
 
     #i = f"{emptyString(1,' ',level)}."
     #i= level
@@ -1105,6 +1325,8 @@ def printParsedLog(context,d):
         _elapsedPrevD = d['timeStamp'][0] - context['prevTimes'][context['prevLevel']][0]
 
     if level<context['prevLevel']:
+        if level<0:
+            a=1
         _elapsedPrevD = d['timeStamp'][0] - context['prevTimes'][level][1]
 
     if _elapsedPrevD <0:
@@ -1204,5 +1426,14 @@ def getFromList(theList,field,value,endsWith=False,delete=True):
         print(e) 
     return None
 
-
+def getFromDebugList(context,values):
+    for line in reversed(context['debugList']):
+        for key in values.keys():
+            if key not in line:
+                break
+            if line[key]!=values[key]:
+                break
+        return line
+    return None    
+                
 
