@@ -188,6 +188,8 @@ def setConfigFile(configFile):
 sfdx_lock = threading.Lock()
 ####CONECTION
 def init(userName_or_orgAlias,connectionName=None):
+    if userName_or_orgAlias  != 'ConnectionLess':
+        glog().info(f"Initializing connection for {userName_or_orgAlias}")
     if userName_or_orgAlias == 'ConnectionLess':
         _initMain(userName_or_orgAlias,None)
         return
@@ -204,7 +206,9 @@ def init(userName_or_orgAlias,connectionName=None):
             return
 
         try:
+            glog().debug(f"Calling sfdx")
             success,obj,outputs = utils.execute_force_org_display(userName_or_orgAlias)
+            glog().debug(f"post sfdx")
 
      #       success,outputs = utils.executeCommandParse(["sfdx","force:org:display","-u", userName_or_orgAlias])
 
@@ -336,7 +340,7 @@ def _gerFolder(name):
     try:
         folder =  getCurrentThreadConnection()[name]
     except Exception as e:
-        if e.args[0]['error'] == 'No default connection for thread':
+        if e.args[0]['error'] == 'No default connection for thread' or e.args[0]['error'] == 'Current Connection is no set.':
             loadConfigData()
             folder = _configData['folders'][name]
         else:
