@@ -65,6 +65,7 @@ def executeCommand(command):
     else:
         list_files = subprocess.run(command,capture_output=True,text=True)
 
+    print(list_files.stderr)
     return list_files
 
 def execute_force_org_display(userName_or_orgAlias):
@@ -478,6 +479,8 @@ def repeatingSequence(objs,field):
 
     repeatingSequences = []
     lastX = 0
+    if objs==None:
+        a=1
     lenObj = len(objs)
     for x,val in enumerate(objs):
         if x <= lastX:
@@ -485,13 +488,15 @@ def repeatingSequence(objs,field):
         firstdelta = None
         lastDelta = 0
         positions = []
+        if 'output' not in objs[x]:
+            oo = objs[x]
         objx = objs[x]['output']
    #     if objs[x][field]=='OrderItem CancelWF --> false':
    #         print(x)
 
         for y in range(x+1,len(objs)):
-            if y>(lenObj-x)/2:
-                break
+         #   if y>(lenObj-x)/2:
+         #       break
             if 'output' not in objs[y]:
                 oo = objs[y]
             objy=objs[y]['output']
