@@ -33,6 +33,14 @@ class Test_Query(unittest.TestCase):
 
         print()
 
+    def test_deleteOrder(self):
+      orderId = '8013O000003a9CoQAI'
+
+      restClient.init('NOSDEV')
+      try:
+        delete = CPQ.deleteCart(orderId)
+      except Exception as e:
+        utils.printException(e)
 
 
 
