@@ -1,4 +1,4 @@
-from InCli.SFAPI import utils,query,restClient
+from InCli.SFAPI import utils,query,restClient,file
 import sys, os
 #python3 -m unittest
 
@@ -96,6 +96,16 @@ class Test_Main(unittest.TestCase):
             print(traceback.format_exc())
             self.assertTrue(1==2)
 
+
+
+    def test_cc_basket(self):
+        try:
+            InCli._main(["-u","NOSQSM","-cc","-basket"])
+        except Exception as e:
+            utils.printException(e)
+            print(traceback.format_exc())
+            self.assertTrue(1==2)
+
     def test_cc_list(self):
         try:
             InCli._main(["-u","NOSDEV","-cc","-list"])
@@ -146,7 +156,8 @@ class Test_Main(unittest.TestCase):
 
     def test_logs_user(self):
         try:
-            InCli._main(["-u","NOSDEV","-logs","-loguser","Alias:ana.r"])
+            res = InCli._main(["-u","NOSDEV","-logs","-loguser","Alias:ana.r"])
+            print()
         except Exception as e:
             utils.printException(e)
             print(traceback.format_exc())
@@ -154,7 +165,8 @@ class Test_Main(unittest.TestCase):
 
     def test_logs_query(self):
         try:
-            InCli._main(["-u","NOSDEV","-logs","-where","Operation='Batch Apex'","-last","10"])
+            res = InCli._main(["-u","NOSDEV","-logs","-where","Operation='Batch Apex'","-last","10"])
+            print()
         except Exception as e:
             utils.printException(e)
             print(traceback.format_exc())
@@ -172,14 +184,15 @@ class Test_Main(unittest.TestCase):
 
     def test_logs_userWrong(self):
         try:
-            InCli._main(["-u","NOSDEV","-logs","-loguser","Alias:xxxx"])
+            res = InCli._main(["-u","NOSDEV","-logs","-loguser","Alias:xxxx"])
         except Exception as e:
             self.assertTrue(e.args[0]['error']=='User with field Alias = xxxx does not exist in the User Object.')
             utils.printException(e)
 
     def test_logs_limit(self):
         try:
-            InCli._main(["-u","NOSDEV","-logs","-limit","2"])
+            res = InCli._main(["-u","NOSDEV","-logs","-limit","2"])
+            self.assertTrue(len(res)==2)
         except Exception as e:
             utils.printException(e)
             print(traceback.format_exc())
@@ -195,13 +208,14 @@ class Test_Main(unittest.TestCase):
    
     def test_log_ID_2(self):
         try:
-            id = '07L3O00000Dh95XUAR'
-      #      id = None
+            id = '07L3O00000DjAwnUAF'
+            env = 'NOSDEV'
+            #id = None
 
             if id == None:
-                restClient.init('NOSDEV')
+                restClient.init(env)
                 id = query.queryField("Select Id FROM ApexLog order by StartTime desc limit 1")
-            InCli._main(["-u","NOSDEV","-logs",id])
+            InCli._main(["-u",env,"-logs",id])
 
         except Exception as e:
             print(e)
@@ -239,7 +253,29 @@ class Test_Main(unittest.TestCase):
 
     def test_log_file(self):
         try:
-            InCli._main(["-u","NOSDEV","-logs","-inputfile",f"/Users/uormaechea/Documents/Dev/python/InCliLib/InCLipkg/test/files/07L3O00000Dgt4sUAB.log"])
+            res = InCli._main(["-u","NOSDEV","-logs","-inputfile",f"/Users/uormaechea/Documents/Dev/python/InCliLib/InCLipkg/test/files/07L3O00000Dgt4sUAB.log"])
+            self.assertTrue(res['lines'][-1] == '18:59:06.100 (4100507463)|CUMULATIVE_PROFILING_END')
+        except Exception as e:
+            utils.printException(e)
+            print(traceback.format_exc())
+            self.assertTrue(1==2)
+
+    def test_log_file_to_file(self):
+        try:
+            restClient.init('NOSDEV')
+            html_name = f"{restClient.logFolder()}07L3O00000Dgt4sUAB.html"
+            txt_name = f"{restClient.logFolder()}07L3O00000Dgt4sUAB.html"
+
+            if file.exists(html_name):
+                file.delete(html_name)
+            if file.exists(txt_name):
+                file.delete(txt_name)
+            self.assertTrue(file.exists(html_name)==False)
+            res = InCli._main(["-u","NOSDEV","-logs","-inputfile",f"/Users/uormaechea/Documents/Dev/python/InCliLib/InCLipkg/test/files/07L3O00000Dgt4sUAB.log","-file"])
+            self.assertTrue(file.exists(html_name)==True)
+            self.assertTrue(file.exists(txt_name)==True)
+
+            html = file.read(f"{restClient.logFolder()}07L3O00000Dgt4sUAB.html")
         except Exception as e:
             utils.printException(e)
             print(traceback.format_exc())
@@ -270,7 +306,7 @@ class Test_Main(unittest.TestCase):
 
     def test_log_last(self):
         try:
-            InCli._main(["-u","NOSDEV","-logs","-last","10"])
+            InCli._main(["-u","NOSQSM","-logs","-last","1000"])
         except Exception as e:
             utils.printException(e)
             print(traceback.format_exc())
@@ -339,6 +375,14 @@ class Test_Main(unittest.TestCase):
             print(traceback.format_exc())
             self.assertTrue(1==2)
 
+    def test_default_weird(self):
+        try:
+            InCli._main(["-h:","InCli","-default:set","u","NOSDEV"])
+        except Exception as e:
+            utils.printException(e)
+            print(traceback.format_exc())
+            self.assertTrue(1==2)
+
     def test_d(self):
         try:
             InCli._main(["-u","NOSDEV","-d"])
